/*
 * Detect a Library for hardware detection
 *
 * Copyright (C) 1998-2000 MandrakeSoft
 * Copyright (C) 2005 Alexandre Dussart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

# define _GNU_SOURCE

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


#include "detect.h"
#include "serial.h"

void print_info(char *mtype, char *mbus, char *device, char *model, char * handler, int dev_n, int detect_lines){
	if (detect_USB==0 && mbus && strcasecmp(mbus,"USB")==0)
		return; // do not detect USB mice.
	if (detect_PS2==0 && mbus && strcasecmp(mbus,"PS2")==0)
		return; // do not detect USB mice.
	if ((strcmp(handler,"kbd")!=0 || (detect_lines & 7) == 7) &&
			mtype!= NULL && mbus != NULL && device != NULL){
		if (extendinfo){
			char bf[50];
			if (dev_n == -1)
				snprintf(bf,49,"%s",device);
			else
				snprintf(bf,49,"/dev/input/%s%d", handler, dev_n);
			printf("Type:%s-%s|Port:%s|Direct:%s|Model:%s\n", mbus, mtype, device, bf, model);
		}else
			printf("Type:%s-%s|Port:%s\n", mbus, mtype, device);
	}
	if(debug && model)
		printf("\t\tFound %s %s %s %s %d %08X\n", model, mtype, mbus, device, dev_n, detect_lines);
}

void do_proc_detect(char *handler, char *gen_device){
	FILE *f;
	char *line=NULL;
	size_t len = 0;
	int in, dev_n=-1, detect_lines=0;
	char model[200];   /* model name of the mouse  */
	char *device=NULL;  /* device name of the mouse */
	char *mtype=NULL;  /* type of the mouse */
	char *mbus=NULL;  /* type of the mouse */
	
	model[0] = '\0';
	
	if(debug)
		printf("\tProbing USB and PS/2 mouse...\n");

	f = fopen(testcases?"testcases.txt":"/proc/bus/input/devices", "r");
	if (f==NULL){
		if (debug) printf("Can't open input(%s) error(%d)%s", testcases?"testcases.txt":"/proc/bus/input/devices", errno, strerror(errno));
		return;
	}
	if ((in=getline(&line, &len, f)) >= 0){
		for (;;) {
			while (in>0 && (line[in-1]=='\r' || line[in-1]=='\n')){
				in--;
				line[in] = '\0';
			}
			
			if (strlen(line) < 4) {
				print_info(mtype, mbus, device, model, handler, dev_n, detect_lines);
				device = mbus = mtype = NULL;
				model[0] = '\0';
				dev_n=-1;
				detect_lines=0;
			}
			
			if (debug) 
				printf("    line: %s\n", line);
			
			//first four flags detect keyboard distinct from IR input hopefully...
			if (strncmp(line, "B: EV=",6)==0) detect_lines |= 1; 
			if (strncmp(line, "B: KEY=",7)==0) detect_lines |= 2;
			if (strncmp(line, "B: LED=",7)==0) detect_lines |= 4;
			//if (strncmp(line, "B: MSC=",7)==0) detect_lines |= 8; //this seems to be PS2 only flag??? its not seen on wireless USB device.

			if (strncmp(line, "N: Name=", 8)==0){
				if (*(line+8)=='\"'){
					int n;
					n = snprintf(model,199,"%s", line+9);
					if (n>0 && model[n-1] == '\"') 
						model[n-1] = '\0';
				}else
					snprintf(model,199,"%s", line+8);
			}
			if (strncmp(line, "P: Phys=", 8)==0){
				if (strstr(line, "usb"))
					mbus="USB";
				else
					mbus="PS2";
			}
			if (strncmp(line,"H: Handlers=", 12)==0){
				char *mouse;
				mouse = strstr(line+12,handler);
				if (mouse != NULL){
					char c;
					mtype=handler;
					device = gen_device;
					c = *(mouse + strlen(handler));
					if (c>='0' && c<='9')
						dev_n=atoi(mouse + strlen(handler));
				}else
					device = "Not Available";
			}
			if ((in=getline(&line, &len, f)) < 0)
				break;
		}
	}
	print_info(mtype, mbus, device, model, handler, dev_n, detect_lines);
	device = mbus = mtype = NULL;
	if (line)
		free(line);
	fclose(f);
}

